@@ -54,6 +54,7 @@ import java.util.zip.ZipOutputStream;
  *
  * @author <a href="mailto:david.lloyd@redhat.com">David M. Lloyd</a>
  * @author Thomas.Diesler@jboss.com
+ * @author <a href="mailto:ropalka@redhat.com">Richard Opalka</a>
  */
 final class JarFileResourceLoader extends AbstractResourceLoader implements IterableResourceLoader {
     private static final String INDEX_FILE = "META-INF/PATHS.LIST";
@@ -81,7 +82,8 @@ final class JarFileResourceLoader extends AbstractResourceLoader implements Iter
         fileOfJar = new File(jarFile.getName());
         this.jarFile = jarFile;
         this.rootName = rootName;
-        final String realPath = relativePath == null ? null : PathUtils.canonicalize(relativePath);
+        String realPath = relativePath == null ? null : PathUtils.canonicalize(relativePath);
+        if (realPath != null && realPath.endsWith("/")) realPath = realPath.substring(0, realPath.length() - 1);
         this.relativePath = realPath;
         try {
             rootUrl = getJarURI(fileOfJar.toURI(), realPath).toURL();
@@ -151,7 +153,9 @@ final class JarFileResourceLoader extends AbstractResourceLoader implements Iter
                     a += res;
                 }
                 // consume remainder so that cert check doesn't fail in case of wonky JARs
-                while (is.read() != -1);
+                while (is.read() != -1) {
+                    //
+                }
                 // done
                 CodeSource codeSource = createCodeSource(entry);
                 is.close();
@@ -235,7 +239,7 @@ final class JarFileResourceLoader extends AbstractResourceLoader implements Iter
             } catch (URISyntaxException x) {
                 throw new IllegalStateException(x);
             }
-            return new JarEntryResource(jarFile, entry, new URL(null, getJarURI(uri, entry.getName()).toString(), (URLStreamHandler) null));
+            return new JarEntryResource(jarFile, entry, relativePath, new URL(null, getJarURI(uri, entry.getName()).toString(), (URLStreamHandler) null));
         } catch (MalformedURLException e) {
             // must be invalid...?  (todo: check this out)
             return null;
@@ -245,8 +249,9 @@ final class JarFileResourceLoader extends AbstractResourceLoader implements Iter
         }
     }
 
-    public Iterator<Resource> iterateResources(final String startPath, final boolean recursive) {
+    public Iterator<Resource> iterateResources(String startPath, final boolean recursive) {
         final JarFile jarFile = this.jarFile;
+        if (relativePath != null) startPath = startPath.equals("") ? relativePath : relativePath + "/" + startPath;
         final String startName = PathUtils.canonicalize(PathUtils.relativize(startPath));
         final Enumeration<JarEntry> entries = jarFile.entries();
         return new Iterator<Resource>() {
@@ -262,7 +267,7 @@ final class JarFileResourceLoader extends AbstractResourceLoader implements Iter
                     if ((recursive ? PathUtils.isChild(startName, name) : PathUtils.isDirectChild(startName, name))) {
                         if (!entry.isDirectory()) {
                             try {
-                                next = new JarEntryResource(jarFile, entry, getJarURI(new File(jarFile.getName()).toURI(), entry.getName()).toURL());
+                                next = new JarEntryResource(jarFile, entry, relativePath, getJarURI(new File(jarFile.getName()).toURI(), entry.getName()).toURL());
                             } catch (Exception ignored) {
                             }
                         }
@@ -325,6 +330,27 @@ final class JarFileResourceLoader extends AbstractResourceLoader implements Iter
         return index;
     }
 
+    @Override
+    public void close() {
+        try {
+            super.close();
+        } finally {
+            try {
+                jarFile.close();
+            } catch (IOException e) {
+                // ignored
+            }
+        }
+    }
+
+    public URI getLocation() {
+        try {
+            return getJarURI(fileOfJar.toURI(), "");
+        } catch (URISyntaxException e) {
+            return null;
+        }
+    }
+
     static void extractJarPaths(final JarFile jarFile, String relativePath,
             final Collection<String> index) {
         index.add("");
@@ -468,7 +494,7 @@ final class JarFileResourceLoader extends AbstractResourceLoader implements Iter
         private final CodeSigner[] codeSigners;
         private final int hashCode;
 
-        public CodeSigners(final CodeSigner[] codeSigners) {
+        CodeSigners(final CodeSigner[] codeSigners) {
             this.codeSigners = codeSigners;
             hashCode = Arrays.hashCode(codeSigners);
         }
