@@ -18,6 +18,7 @@
 
 package org.jboss.modules;
 
+import java.lang.reflect.InvocationTargetException;
 import java.security.PermissionCollection;
 import java.security.ProtectionDomain;
 import java.util.IdentityHashMap;
@@ -39,7 +40,7 @@ import java.util.HashMap;
 import java.util.List;
 import java.util.Map;
 import java.util.Set;
-import java.util.concurrent.atomic.AtomicReferenceFieldUpdater;
+import java.util.concurrent.atomic.AtomicReference;
 
 /**
  * A module classloader.  Instances of this class implement the complete view of classes and resources available in a
@@ -69,7 +70,7 @@ public class ModuleClassLoader extends ConcurrentClassLoader {
     private final Module module;
     private final ClassFileTransformer transformer;
 
-    private volatile Paths<ResourceLoader, ResourceLoaderSpec> paths;
+    private final AtomicReference<Paths<ResourceLoader, ResourceLoaderSpec>> paths = new AtomicReference<>(Paths.<ResourceLoader, ResourceLoaderSpec>none());
 
     private final LocalLoader localLoader = new IterableLocalLoader() {
         public Class<?> loadClassLocal(final String name, final boolean resolve) {
@@ -104,14 +105,6 @@ public class ModuleClassLoader extends ConcurrentClassLoader {
         }
     };
 
-    private static final AtomicReferenceFieldUpdater<ModuleClassLoader, Paths<ResourceLoader, ResourceLoaderSpec>> pathsUpdater
-            = unsafeCast(AtomicReferenceFieldUpdater.newUpdater(ModuleClassLoader.class, Paths.class, "paths"));
-
-    @SuppressWarnings({ "unchecked" })
-    private static <A, B> AtomicReferenceFieldUpdater<A, B> unsafeCast(AtomicReferenceFieldUpdater<?, ?> updater) {
-        return (AtomicReferenceFieldUpdater<A, B>) updater;
-    }
-
     /**
      * Construct a new instance.
      *
@@ -119,7 +112,7 @@ public class ModuleClassLoader extends ConcurrentClassLoader {
      */
     protected ModuleClassLoader(final Configuration configuration) {
         module = configuration.getModule();
-        paths = new Paths<ResourceLoader, ResourceLoaderSpec>(configuration.getResourceLoaders(), Collections.<String, List<ResourceLoader>>emptyMap());
+        paths.lazySet(new Paths<>(configuration.getResourceLoaders(), Collections.<String, List<ResourceLoader>>emptyMap()));
         final AssertionSetting setting = configuration.getAssertionSetting();
         if (setting != AssertionSetting.INHERIT) {
             setDefaultAssertionStatus(setting == AssertionSetting.ENABLED);
@@ -134,7 +127,7 @@ public class ModuleClassLoader extends ConcurrentClassLoader {
      *  before the calling thread
      */
     boolean recalculate() {
-        final Paths<ResourceLoader, ResourceLoaderSpec> paths = this.paths;
+        final Paths<ResourceLoader, ResourceLoaderSpec> paths = this.paths.get();
         return setResourceLoaders(paths, paths.getSourceList(NO_RESOURCE_LOADERS));
     }
 
@@ -146,7 +139,7 @@ public class ModuleClassLoader extends ConcurrentClassLoader {
      *  before the calling thread
      */
     boolean setResourceLoaders(final ResourceLoaderSpec[] resourceLoaders) {
-        return setResourceLoaders(paths, resourceLoaders);
+        return setResourceLoaders(paths.get(), resourceLoaders);
     }
 
     private boolean setResourceLoaders(final Paths<ResourceLoader, ResourceLoaderSpec> paths, final ResourceLoaderSpec[] resourceLoaders) {
@@ -167,7 +160,7 @@ public class ModuleClassLoader extends ConcurrentClassLoader {
                 }
             }
         }
-        return pathsUpdater.compareAndSet(this, paths, new Paths<ResourceLoader, ResourceLoaderSpec>(resourceLoaders, allPaths));
+        return this.paths.compareAndSet(paths, new Paths<>(resourceLoaders, allPaths));
     }
 
     /**
@@ -202,7 +195,19 @@ public class ModuleClassLoader extends ConcurrentClassLoader {
 
         log.trace("Class %s not found from %s", className, module);
 
-        throw new ClassNotFoundException(className + " from [" + module + "]");
+        throw new ClassNotFoundException(getClassNotFoundExceptionMessage(className, module));
+    }
+
+    /**
+     * Returns an exception message used when producing instances of ClassNotFoundException. This can be overridden
+     * by subclasses to customise the error message.
+     *
+     * @param className the name of the class which is missing
+     * @param fromModule the module from which the class could not be found
+     * @return an exception message used when producing instances of ClassNotFoundException
+     */
+    protected String getClassNotFoundExceptionMessage(String className, Module fromModule){
+        return className + " from [" + fromModule + "]";
     }
 
     /**
@@ -239,7 +244,7 @@ public class ModuleClassLoader extends ConcurrentClassLoader {
             return loadedClass;
         }
 
-        final Map<String, List<ResourceLoader>> paths = this.paths.getAllPaths();
+        final Map<String, List<ResourceLoader>> paths = this.paths.get().getAllPaths();
 
         log.trace("Loading class %s locally from %s", className, module);
 
@@ -302,7 +307,7 @@ public class ModuleClassLoader extends ConcurrentClassLoader {
      */
     Resource loadResourceLocal(final String root, final String name) {
 
-        final Map<String, List<ResourceLoader>> paths = this.paths.getAllPaths();
+        final Map<String, List<ResourceLoader>> paths = this.paths.get().getAllPaths();
 
         final String path = Module.pathOf(name);
 
@@ -328,7 +333,7 @@ public class ModuleClassLoader extends ConcurrentClassLoader {
      * @return the list of resources
      */
     public List<Resource> loadResourceLocal(final String name) {
-        final Map<String, List<ResourceLoader>> paths = this.paths.getAllPaths();
+        final Map<String, List<ResourceLoader>> paths = this.paths.get().getAllPaths();
 
         final String path = Module.pathOf(name);
 
@@ -432,16 +437,20 @@ public class ModuleClassLoader extends ConcurrentClassLoader {
                 newClass = doDefineOrLoadClass(name, bytes, 0, bytes.length, protectionDomain);
                 module.getModuleLoader().addClassLoadTime(Metrics.getCurrentCPUTime() - start);
                 log.classDefined(name, module);
-            } catch (NoClassDefFoundError e) {
+            } catch (LinkageError e) {
                 // Prepend the current class name, so that transitive class definition issues are clearly expressed
-                final LinkageError ne = new LinkageError("Failed to link " + name.replace('.', '/') + " (" + module + ")");
-                ne.initCause(e);
+                Error ne;
+                try {
+                    final String oldMsg = e.getMessage();
+                    final String newMsg = "Failed to link " + name.replace('.', '/') + " (" + module + ")";
+                    ne = e.getClass().getConstructor(String.class).newInstance(oldMsg == null || oldMsg.isEmpty() ? newMsg : newMsg + ": " + oldMsg);
+                } catch (InstantiationException | NoSuchMethodException | InvocationTargetException | IllegalAccessException ignored) {
+                    // just throw the original
+                    throw e;
+                }
                 throw ne;
             }
-        } catch (Error e) {
-            log.classDefineFailed(e, name, module);
-            throw e;
-        } catch (RuntimeException e) {
+        } catch (Error | RuntimeException e) {
             log.classDefineFailed(e, name, module);
             throw e;
         }
@@ -509,7 +518,7 @@ public class ModuleClassLoader extends ConcurrentClassLoader {
         final ModuleLogger log = Module.log;
         log.trace("Attempting to load native library %s from %s", libname, module);
 
-        for (ResourceLoaderSpec loader : paths.getSourceList(NO_RESOURCE_LOADERS)) {
+        for (ResourceLoaderSpec loader : paths.get().getSourceList(NO_RESOURCE_LOADERS)) {
             final String library = loader.getResourceLoader().getLibrary(libname);
             if (library != null) {
                 return library;
@@ -560,7 +569,7 @@ public class ModuleClassLoader extends ConcurrentClassLoader {
     }
 
     Set<String> getPaths() {
-        return paths.getAllPaths().keySet();
+        return paths.get().getAllPaths().keySet();
     }
 
     /** {@inheritDoc} */
@@ -634,7 +643,7 @@ public class ModuleClassLoader extends ConcurrentClassLoader {
     }
 
     ResourceLoader[] getResourceLoaders() {
-        final ResourceLoaderSpec[] specs = paths.getSourceList(NO_RESOURCE_LOADERS);
+        final ResourceLoaderSpec[] specs = paths.get().getSourceList(NO_RESOURCE_LOADERS);
         final int length = specs.length;
         final ResourceLoader[] loaders = new ResourceLoader[length];
         for (int i = 0; i < length; i++) {
@@ -664,7 +673,7 @@ public class ModuleClassLoader extends ConcurrentClassLoader {
         } else {
             filter = PathFilters.is(realStartName);
         }
-        final Map<String, List<ResourceLoader>> paths = this.paths.getAllPaths();
+        final Map<String, List<ResourceLoader>> paths = this.paths.get().getAllPaths();
         final Iterator<Map.Entry<String, List<ResourceLoader>>> iterator = paths.entrySet().iterator();
         return new Iterator<Resource>() {
 
@@ -730,7 +739,7 @@ public class ModuleClassLoader extends ConcurrentClassLoader {
      * @return the set of local paths
      */
     public final Set<String> getLocalPaths() {
-        return Collections.unmodifiableSet(paths.getAllPaths().keySet());
+        return Collections.unmodifiableSet(paths.get().getAllPaths().keySet());
     }
 
     /**
